@@ -5,7 +5,6 @@ import java.util.Arrays;
 import java.util.Calendar;
 import java.util.Collections;
 import java.util.Comparator;
-import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
@@ -14,15 +13,18 @@ import java.util.concurrent.ConcurrentHashMap;
 
 import javax.servlet.http.HttpSession;
 
-import org.gcube.datacatalogue.ckanutillibrary.server.DataCatalogue;
-import org.gcube.datacatalogue.ckanutillibrary.server.DataCatalogueFactory;
-import org.gcube.datacatalogue.ckanutillibrary.server.utils.SessionCatalogueAttributes;
-import org.gcube.datacatalogue.ckanutillibrary.server.utils.CatalogueUtilMethods;
-import org.gcube.datacatalogue.ckanutillibrary.shared.ResourceBean;
-import org.gcube.datacatalogue.ckanutillibrary.shared.RolesCkanGroupOrOrg;
+import org.gcube.common.portal.PortalContext;
+import org.gcube.common.storagehubwrapper.server.StorageHubWrapper;
+import org.gcube.common.storagehubwrapper.server.tohl.Workspace;
+import org.gcube.datacatalogue.utillibrary.server.DataCatalogue;
+import org.gcube.datacatalogue.utillibrary.server.DataCatalogueFactory;
+import org.gcube.datacatalogue.utillibrary.server.utils.CatalogueUtilMethods;
+import org.gcube.datacatalogue.utillibrary.server.utils.SessionCatalogueAttributes;
+import org.gcube.datacatalogue.utillibrary.shared.ResourceBean;
+import org.gcube.datacatalogue.utillibrary.shared.RolesCkanGroupOrOrg;
+import org.gcube.datacatalogue.utillibrary.shared.jackan.model.CkanGroup;
+import org.gcube.datacatalogue.utillibrary.shared.jackan.model.CkanLicense;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.client.CKanPublisherService;
-import org.gcube.portlets.widgets.ckandatapublisherwidget.server.threads.AssociationToGroupAndNotifyThread;
-import org.gcube.portlets.widgets.ckandatapublisherwidget.server.threads.WritePostCatalogueManagerThread;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.server.utils.CatalogueRoleManager;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.server.utils.DiscoverTagsList;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.server.utils.GenericUtils;
@@ -41,15 +43,14 @@ import org.gcube.vomanagement.usermanagement.impl.LiferayGroupManager;
 import org.gcube.vomanagement.usermanagement.impl.LiferayUserManager;
 import org.gcube.vomanagement.usermanagement.model.GCubeUser;
 import org.geojson.GeoJsonObject;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import com.fasterxml.jackson.databind.ObjectMapper;
 import com.google.gwt.user.server.rpc.RemoteServiceServlet;
-import com.liferay.portal.kernel.log.Log;
-import com.liferay.portal.kernel.log.LogFactoryUtil;
 import com.liferay.portal.service.UserLocalServiceUtil;
 
-import eu.trentorise.opendata.jackan.model.CkanGroup;
-import eu.trentorise.opendata.jackan.model.CkanLicense;
+
 
 /**
  * Server side of the data publisher.
@@ -61,7 +62,7 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 
 	// Logger
 	//private static final org.slf4j.Logger logger = LoggerFactory.getLogger(CKANPublisherServicesImpl.class);
-	private static final Log logger = LogFactoryUtil.getLog(CKANPublisherServicesImpl.class);
+	private static final Logger logger = LoggerFactory.getLogger(CKANPublisherServicesImpl.class);
 	private static final String ITEM_URL_FIELD = "Item URL";
 	private static final String SYS_TYPE = "system:type";
 	private static final String TAGS_VOCABULARY_KEY = "TAGS_VOCABULARY";
@@ -69,17 +70,31 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 	// map <orgName, scope>
 	private ConcurrentHashMap<String, String> mapOrganizationScope = new ConcurrentHashMap<String, String>();
 
+	
 	/**
-	 * Retrieve an instance of the library for the scope
+	 * Dev mode set contexts.
+	 */
+	private void devModeSetContexts() {
+		if (!isWithinPortal()) {
+			logger.info("DETECTED DEV MODE");
+			GenericUtils.getCurrentContext(getThreadLocalRequest(), true);
+			GenericUtils.getCurrentToken(getThreadLocalRequest(), true);
+		}
+		
+	}
+
+	/**
+	 * Retrieve an instance of the library for the scope.
+	 *
 	 * @param scope if it is null it is evaluated from the session
-	 * @return
+	 * @return the catalogue
 	 */
 	public DataCatalogue getCatalogue(String scope){
 
 		DataCatalogue instance = null;
 		String scopeInWhichDiscover = null;
 		try{
-			scopeInWhichDiscover = (scope != null && !scope.isEmpty()) ? scope : GenericUtils.getCurrentContext(getThreadLocalRequest(), false);
+			scopeInWhichDiscover = scope != null && !scope.isEmpty() ? scope : GenericUtils.getCurrentContext(getThreadLocalRequest(), false);
 			logger.debug("Discovering ckan instance into scope " + scopeInWhichDiscover);
 			instance = DataCatalogueFactory.getFactory().getUtilsPerScope(scopeInWhichDiscover);
 		}catch(Exception e){
@@ -89,11 +104,13 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 	}
 
 	/**
-	 * Retrieve the list of organizations in which the user can publish (roles ADMIN/EDITOR)
-	 * @param username
+	 * Retrieve the list of organizations in which the user can publish (roles ADMIN/EDITOR).
+	 *
+	 * @param username the username
+	 * @param scope the scope
 	 * @return the list of organizations
-	 * @throws GroupRetrievalFault 
-	 * @throws UserManagementSystemException 
+	 * @throws UserManagementSystemException the user management system exception
+	 * @throws GroupRetrievalFault the group retrieval fault
 	 */
 	private List<OrganizationBean> getUserOrganizationsListAdmin(String username, String scope) throws UserManagementSystemException, GroupRetrievalFault {
 
@@ -108,7 +125,9 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 			logger.info("List of organizations was into session " + orgsName);
 		}
 		else{
-			CatalogueRoleManager.getHighestRole(scope, username, GenericUtils.getGroupFromScope(scope).getGroupName(), this, orgsName);
+			String gatewayURL = GenericUtils.getGatewayClientHostname(getThreadLocalRequest());
+			logger.info("The Gateway URL is: "+gatewayURL);
+			CatalogueRoleManager.getHighestRole(scope, username, GenericUtils.getGroupFromScope(scope).getGroupName(), this, orgsName, gatewayURL);
 			httpSession.setAttribute(keyPerScope, orgsName);
 			logger.info("Organizations name for user " + username + " has been saved into session " + orgsName);
 		}
@@ -117,26 +136,41 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 	}
 
 	/**
-	 * Online or in development mode?
+	 * Online or in development mode?.
+	 *
 	 * @return true if you're running into the portal, false if in development
 	 */
 	private boolean isWithinPortal() {
 		try {
 			UserLocalServiceUtil.getService();
 			return true;
-		} 
-		catch (com.liferay.portal.kernel.bean.BeanLocatorException ex) {			
+		}
+		catch (com.liferay.portal.kernel.bean.BeanLocatorException ex) {
 			logger.trace("Development Mode ON");
 			return false;
-		}			
+		}
+	}
+	
+	/**
+	 * Gets the workspace from storage hub.
+	 *
+	 * @return the workspace from storage hub
+	 * @throws Exception
+	 *             the exception
+	 */
+	protected Workspace getWorkspaceFromStorageHub() throws Exception {
+		GCubeUser user = PortalContext.getConfiguration().getCurrentUser(this.getThreadLocalRequest());
+		StorageHubWrapper storageHubWrapper = WorkspaceUtils.getStorageHubWrapper(this.getThreadLocalRequest(), null, user);
+		return storageHubWrapper.getWorkspace();
 	}
 
 	/**
 	 * Find a license id given the license text.
-	 * @param chosenLicense
-	 * @return
+	 *
+	 * @param chosenLicense the chosen license
+	 * @return the string
 	 */
-	private String findLicenseIdByLicense(String chosenLicense) {	
+	private String findLicenseIdByLicense(String chosenLicense) {
 
 		// get scope from client url
 		String scope = GenericUtils.getScopeFromClientUrl(getThreadLocalRequest());
@@ -144,6 +178,11 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 
 	}
 
+	/**
+	 * Gets the licenses.
+	 *
+	 * @return the licenses
+	 */
 	@Override
 	public List<LicenseBean> getLicenses() {
 
@@ -152,6 +191,13 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 		String scope = GenericUtils.getScopeFromClientUrl(getThreadLocalRequest());
 		logger.info("Request for CKAN licenses for scope " + scope);
 		String keyPerScope = CatalogueUtilMethods.concatenateSessionKeyScope(SessionCatalogueAttributes.CKAN_LICENSES_KEY, scope);
+		
+//		if(!isWithinPortal()){
+//			logger.info("DEV MODE returning funny licenses...");
+//			List<LicenseBean> licenses =  new ArrayList<LicenseBean>();
+//			licenses.add(new LicenseBean("AFL-3.0", "https://opensource.org/licenses/AFL-3.0"));
+//			return licenses;
+//		}
 
 		List<LicenseBean> licensesBean = null;
 		if(httpSession.getAttribute(keyPerScope) != null){
@@ -177,7 +223,15 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 
 		return licensesBean;
 	}
+	
 
+	/**
+	 * Gets the dataset bean.
+	 *
+	 * @param folderId the folder id
+	 * @return the dataset bean
+	 * @throws Exception the exception
+	 */
 	@Override
 	public DatasetBean getDatasetBean(String folderId) throws Exception{
 
@@ -192,7 +246,6 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 				String scope = GenericUtils.getScopeFromClientUrl(getThreadLocalRequest());
 
 				logger.debug("Scope recovered from session is " + scope);
-
 				logger.debug("Request dataset metadata bean for folder with id " + folderId
 						+ " whose owner is " + userName);
 
@@ -216,7 +269,8 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 
 				// if the request comes from the workspace
 				if(folderId != null && !folderId.isEmpty()){
-					WorkspaceUtils.handleWorkspaceResources(folderId, userName, bean);
+					Workspace workspace = getWorkspaceFromStorageHub();
+					WorkspaceUtils.toWorkspaceResource(folderId, userName, bean, workspace);
 				}
 
 			}catch(Exception e){
@@ -235,16 +289,23 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 				bean.setVersion(1);
 				String onlyAlphanumeric = "test-creation-blablabla".replaceAll("[^A-Za-z0-9]", "");
 				bean.setTitle(onlyAlphanumeric + Calendar.getInstance().getTimeInMillis());
-				bean.setAuthorName("Costantino");
-				bean.setAuthorSurname("Perciante");
-				bean.setAuthorEmail("costantino.perciante@isti.cnr.it");
-				bean.setMaintainer("Costantino Perciante");
-				bean.setMaintainerEmail("costantino.perciante@isti.cnr.it");
-				bean.setOrganizationList(Arrays.asList(new OrganizationBean("NextNext", "nextnext", true)));
+				bean.setAuthorName("Francesco");
+				bean.setAuthorSurname("Mangiacrapa");
+				bean.setAuthorEmail("francesco.mangiacrapa@isti.cnr.it");
+				bean.setMaintainer("Francesco Mangiacrapa");
+				bean.setMaintainerEmail("francesco.mangiacrapa@isti.cnr.it");
+
+				//UPDATED By Francesco
+				String scope = GenericUtils.getCurrentContext(this.getThreadLocalRequest(), false);
+				String vreName = scope.substring(scope.lastIndexOf("/")+1,scope.length());
+				logger.debug("In dev mode using the scope: "+scope+" and VRE name: "+vreName);
+				bean.setOrganizationList(Arrays.asList(new OrganizationBean(vreName, vreName.toLowerCase(), true)));
+
 				bean.setOwnerIdentifier(userName);
 
 				if(folderId != null && !folderId.isEmpty()){
-					WorkspaceUtils.handleWorkspaceResources(folderId, userName, bean);
+					Workspace workspace = getWorkspaceFromStorageHub();
+					WorkspaceUtils.toWorkspaceResource(folderId, userName, bean, workspace);
 				}
 			}catch(Exception e){
 				logger.error("Error while building bean into dev mode", e);
@@ -257,14 +318,15 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 	}
 
 	/**
-	 * Discover from the IS the vocabulary of tags for this scope, if present
+	 * Discover from the IS the vocabulary of tags for this scope, if present.
+	 *
+	 * @param context the context
 	 * @return a list of tags vocabulary
-	 * @throws Exception 
 	 */
 	private List<String> discoverTagsVocabulary(String context){
 
 		logger.debug("Looking for vocabulary of tags in this context " + context);
-		String keyPerVocabulary = CatalogueUtilMethods.concatenateSessionKeyScope(TAGS_VOCABULARY_KEY, context); 
+		String keyPerVocabulary = CatalogueUtilMethods.concatenateSessionKeyScope(TAGS_VOCABULARY_KEY, context);
 
 		List<String> vocabulary = (List<String>) getThreadLocalRequest().getSession().getAttribute(keyPerVocabulary);
 		if(vocabulary == null){
@@ -278,15 +340,30 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 		return vocabulary;
 	}
 
+	/**
+	 * Gets the tags for organization.
+	 *
+	 * @param orgName the org name
+	 * @return the tags for organization
+	 * @throws Exception the exception
+	 */
 	@Override
 	public List<String> getTagsForOrganization(String orgName) throws Exception{
 		return discoverTagsVocabulary(getScopeFromOrgName(orgName));
 	}
 
+	/**
+	 * Creates the C kan dataset.
+	 *
+	 * @param toCreate the to create
+	 * @return the dataset bean
+	 * @throws Exception the exception
+	 */
 	@Override
 	public DatasetBean createCKanDataset(DatasetBean toCreate) throws Exception{
 
 		try{
+			devModeSetContexts();
 			logger.info("Request for creating a dataset with these information " + toCreate);
 			String userName = GenericUtils.getCurrentUser(getThreadLocalRequest()).getUsername();
 			String title = toCreate.getTitle();
@@ -311,70 +388,55 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 			// get the list of resources and convert to ResourceBean
 			List<ResourceBean> resources = null;
 			ResourceElementBean resourcesToAdd = toCreate.getResourceRoot();
-
-			// we need to copy such resource in the .catalogue area of the user's ws
+			
+			//converting to resources to be added
 			if(resourcesToAdd != null){
-				resources = WorkspaceUtils.copyResourcesToUserCatalogueArea(toCreate.getId(), userName, toCreate);
+				Workspace workspace = getWorkspaceFromStorageHub();
+				resources = WorkspaceUtils.toResources(toCreate, workspace, userName);
 			}
 
 			logger.debug("The user wants to publish in organization with name " + organizationNameOrId);
 			String scope = getScopeFromOrgName(organizationNameOrId);
 			DataCatalogue utils = getCatalogue(scope);
-			String userApiKey = utils.getApiKeyFromUsername(userName);
-
-			String datasetId = utils.createCKanDatasetMultipleCustomFields
-					(userApiKey, title, null, organizationNameOrId, author, authorMail, maintainer, 
-							maintainerMail, version, description, licenseId, listOfTags, customFields, resources, setPublic);
+			
+			if (!isWithinPortal()) {
+				logger.debug("Should be added:");
+				for (String key : customFields.keySet()) {
+					logger.debug("Custom field with key: "+key+", value: "+customFields.get(key));
+				}
+			}
+			
+			String datasetId = utils.createCkanDatasetMultipleCustomFields(userName, 
+					title, 
+					null, 
+					organizationNameOrId, 
+					author, 
+					authorMail, 
+					maintainer, 
+					maintainerMail, 
+					version, 
+					description, 
+					licenseId, 
+					listOfTags, 
+					customFields, 
+					resources, 
+					setPublic, 
+					true, 
+					true);
 
+			
 			if(datasetId != null){
 
 				logger.info("Dataset created!");
 				toCreate.setId(datasetId);
 
 				// retrieve the url
-				String datasetUrl =  utils.getUnencryptedUrlFromDatasetIdOrName(datasetId);
-				toCreate.setSource(datasetUrl);
-
-				// add also this information as custom field
-				Map<String, List<String>> addField = new HashMap<String, List<String>>();
-				addField.put(ITEM_URL_FIELD, Arrays.asList(datasetUrl));
-				utils.patchProductCustomFields(datasetId, userApiKey, addField, false);
-
-				// start a thread that will associate this dataset with the group
-				if(/*toCreate.getChosenType() != null ||*/ toCreate.getGroups() != null){
-
-					AssociationToGroupAndNotifyThread threadAssociationToGroup = 
-							new AssociationToGroupAndNotifyThread(
-									toCreate.getGroups(),
-									toCreate.getGroupsForceCreation(),
-									null, //toCreate.getChosenType(), TODO
-									datasetUrl,
-									datasetId,
-									toCreate.getTitle(),
-									GenericUtils.getCurrentUser(getThreadLocalRequest()).getFullname(),
-									userName, 
-									utils, 
-									organizationNameOrId,
-									getThreadLocalRequest()
-									);
-					threadAssociationToGroup.start();
-
-				}
-
-				// launch notification thread
-				WritePostCatalogueManagerThread threadWritePost = 
-						new WritePostCatalogueManagerThread(
-								userName,
-								scope, 
-								toCreate.getTitle(), 
-								datasetUrl, 
-								false, // send notification to other people 
-								toCreate.getTags(), 
-								GenericUtils.getCurrentUser(getThreadLocalRequest()).getFullname(),
-								GenericUtils.getCurrentClientUrl(getThreadLocalRequest())
-								);
-				threadWritePost.start();
-
+				//String datasetUrl =  utils.getUnencryptedUrlFromDatasetIdOrName(datasetId);
+				//toCreate.setSource(datasetUrl);
+				
+				//#23491 Building the go to the item to "Catalogue Portlet URL" (instead of URI Resolver URL)
+				String catalogueURL =  utils.getPortletUrl();
+				toCreate.setSource(String.format("%s?path=/dataset/%s", catalogueURL,datasetId));
 				return toCreate;
 			}else{
 				logger.error("Failed to create the dataset");
@@ -382,84 +444,91 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 
 		}catch(Exception e){
 			logger.error("Error while creating item ", e);
-			throw new Exception("An error occurred while creating the item " + e.getMessage());
+			throw new Exception(e.getMessage());
 		}
 
 		return null;
 
 	}
-
+	
+	/**
+	 * Adds the resource to dataset.
+	 *
+	 * @param resource the resource
+	 * @param datasetId the dataset id
+	 * @return the resource element bean
+	 * @throws Exception the exception
+	 */
 	@Override
 	public ResourceElementBean addResourceToDataset(ResourceElementBean resource, String datasetId) throws Exception{
+		logger.info("called addResourceToDataset");
+		devModeSetContexts();
 		String username = GenericUtils.getCurrentUser(getThreadLocalRequest()).getUsername();
-
 		logger.debug("Incoming request for creating new resource for dataset with id " + datasetId + " and organization name of the dataset is " + resource.getOrganizationNameDatasetParent());
 		logger.debug("Owner is " + username + " and resource is " + resource);
 
-		if(!isWithinPortal()){
+		ResourceBean resourceBean = new ResourceBean(
+				resource.getUrl(),
+				resource.getName(),
+				resource.getDescription(),
+				null,
+				username,
+				datasetId,
+				null);
 
-			logger.warn("Running outside the portal");
+		// get the scope in which we should discover the ckan instance given the organization name in which the dataset was created
+		String scope = getScopeFromOrgName(resource.getOrganizationNameDatasetParent());
+		DataCatalogue catalogue = getCatalogue(scope);
+		String resourceId = catalogue.addResourceToDataset(resourceBean, resource.getOrganizationNameDatasetParent(), username);
+
+		if(resourceId != null){
+			logger.info("Resource  " + resource.getName() + " is now available");
+			// set its id and turn it to the client
+			resource.setOriginalIdInWorkspace(resourceId);
 			return resource;
+		}
 
-		}else{
-
-
-			ResourceBean resourceBean = new ResourceBean(
-					resource.getUrl(), 
-					resource.getName(), 
-					resource.getDescription(), 
-					null, 
-					username, 
-					datasetId,
-					null);
+		logger.debug("No resource created");
+		return null;
+	}
 
+	/**
+	 * Delete resource from dataset.
+	 *
+	 * @param resource the resource
+	 * @return true, if successful
+	 * @throws Exception the exception
+	 */
+	@Override
+	public boolean deleteResourceFromDataset(ResourceElementBean resource) throws Exception{
+		logger.debug("Request for deleting resource " + resource);
+		boolean deleted = false;
+		devModeSetContexts();
+		try{
 			// get the scope in which we should discover the ckan instance given the organization name in which the dataset was created
 			String scope = getScopeFromOrgName(resource.getOrganizationNameDatasetParent());
 			DataCatalogue catalogue = getCatalogue(scope);
-			String resourceId = catalogue.addResourceToDataset(resourceBean, catalogue.getApiKeyFromUsername(username));
-
-			if(resourceId != null){
-				logger.debug("Resource  " + resource.getName() + " is now available");
-				// set its id and turn it to the client
-				resource.setOriginalIdInWorkspace(resourceId);
-				return resource;
-			}
-
-			logger.debug("No resource created");
-			return null;
-		}
-	}
-
-	@Override
-	public boolean deleteResourceFromDataset(ResourceElementBean resource) throws Exception{
-
-		logger.debug("Request for deleting resource " + resource);
-		boolean deleted = false;
-
-		if(!isWithinPortal()){
-			logger.warn("Running outside the portal");
-			return deleted;
-		}else{
-
 			String username = GenericUtils.getCurrentUser(getThreadLocalRequest()).getUsername();
-			try{
-				// get the scope in which we should discover the ckan instance given the organization name in which the dataset was created
-				String scope = getScopeFromOrgName(resource.getOrganizationNameDatasetParent());
-				DataCatalogue catalogue = getCatalogue(scope);
-				deleted = catalogue.
-						deleteResourceFromDataset(resource.getOriginalIdInWorkspace(), catalogue.getApiKeyFromUsername(username));
-				if(deleted){
-					logger.info("Resource described by " + resource + " deleted");
-				}else
-					logger.error("Resource described by " + resource + " NOT deleted");
-			}catch(Exception e){
-				logger.error("Error while trying to delete resource described by " + resource, e);
-				throw new Exception("Error while trying to delete resource." + e.getMessage());
-			}
-			return deleted;
+			deleted = catalogue.deleteResourceFromDataset(resource.getOriginalIdInWorkspace(),username);
+			if(deleted){
+				logger.info("Resource described by " + resource + " deleted");
+			}else
+				logger.error("Resource described by " + resource + " NOT deleted");
+		}catch(Exception e){
+			logger.error("Error while trying to delete resource described by " + resource, e);
+			throw new Exception("Error while trying to delete resource." + e.getMessage());
 		}
+		return deleted;
+	
 	}
 
+	/**
+	 * Gets the profiles.
+	 *
+	 * @param orgName the org name
+	 * @return the profiles
+	 * @throws Exception the exception
+	 */
 	@Override
 	public List<MetaDataProfileBean> getProfiles(String orgName) throws Exception{
 
@@ -477,6 +546,14 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 		return toReturn;
 	}
 
+	/**
+	 * Dataset id already exists.
+	 *
+	 * @param title the title
+	 * @param orgName the org name
+	 * @return true, if successful
+	 * @throws Exception the exception
+	 */
 	@Override
 	public boolean datasetIdAlreadyExists(String title, String orgName) throws Exception{
 		if(title == null || title.isEmpty())
@@ -498,18 +575,23 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 	 */
 	private String getScopeFromOrgName(String orgName){
 
-		logger.debug("Request for scope related to orgName " + orgName + "[ map that will be used is " + mapOrganizationScope.toString() + " ]");
+		logger.info("Request for scope related to orgName " + orgName + " [map that will be used is " + mapOrganizationScope.toString() + "]");
 		if(orgName == null || orgName.isEmpty())
 			throw new IllegalArgumentException("orgName cannot be empty or null!");
 
 		String toReturn = null;
 
 		if(isWithinPortal()){
-			if(mapOrganizationScope.containsKey(orgName))
+			if(mapOrganizationScope.containsKey(orgName)) {
 				toReturn = mapOrganizationScope.get(orgName);
-			else{
+			}else{
 				try{
 					String evaluatedScope = GenericUtils.retrieveScopeFromOrganizationName(orgName);
+					//see #20801
+					if(evaluatedScope==null || evaluatedScope.isEmpty()) {
+						logger.warn("Scope detected for OrganizationName: "+orgName+" is null or empty, skipping filling 'mapOrganizationScope' and returning null");
+						return toReturn;
+					}
 					mapOrganizationScope.put(orgName, evaluatedScope);
 					toReturn = evaluatedScope;
 				}catch(Exception e){
@@ -517,20 +599,30 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 				}
 			}
 		}else{
-			toReturn = "/gcube/devNext/NextNext";
+			//UPDATED By FRANCESCO
+			toReturn = GenericUtils.getCurrentContext(this.getThreadLocalRequest(), false);
 			mapOrganizationScope.put(orgName, toReturn);
 		}
-		logger.debug("Returning scope " + toReturn);
+		logger.info("Returning scope " + toReturn);
 		return toReturn;
 	}
 
+	/**
+	 * Gets the user groups.
+	 *
+	 * @param orgName the org name
+	 * @return the user groups
+	 */
 	@Override
 	public List<OrganizationBean> getUserGroups(String orgName) {
 
 		List<OrganizationBean> toReturn = new ArrayList<OrganizationBean>();
 
 		if(isWithinPortal()){
-			String username = GenericUtils.getCurrentUser(getThreadLocalRequest()).getUsername();
+			GCubeUser user = GenericUtils.getCurrentUser(getThreadLocalRequest());
+			String username = null;
+			if(user!=null)
+				username = user.getUsername();
 
 			logger.debug("Request for user " + username + " groups. Organization name is "  + orgName);
 
@@ -546,17 +638,25 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 				logger.info("Found user's groups in session " + toReturn);
 			}else{
 
-				DataCatalogue catalogue = getCatalogue(scope);
-				String apiKey = catalogue.getApiKeyFromUsername(username);
-				Map<String, Map<CkanGroup, RolesCkanGroupOrOrg>> mapRoleGroup = catalogue.getUserRoleByGroup(username, apiKey);
-				Set<Entry<String, Map<CkanGroup, RolesCkanGroupOrOrg>>> set = mapRoleGroup.entrySet();
-				for (Entry<String, Map<CkanGroup, RolesCkanGroupOrOrg>> entry : set) {
-					Set<Entry<CkanGroup, RolesCkanGroupOrOrg>> subSet = entry.getValue().entrySet();
-					for (Entry<CkanGroup, RolesCkanGroupOrOrg> subEntry : subSet) {
-						toReturn.add(new OrganizationBean(subEntry.getKey().getTitle(), subEntry.getKey().getName(), false));	
-					}
+				//Fixing Incident #12563
+				try{
+					DataCatalogue catalogue = getCatalogue(scope);
+					//Fixing Incident #12563
+					if(username!=null && !username.isEmpty()){
+						Map<String, Map<CkanGroup, RolesCkanGroupOrOrg>> mapRoleGroup = catalogue.getUserRoleByGroup(username);
+						Set<Entry<String, Map<CkanGroup, RolesCkanGroupOrOrg>>> set = mapRoleGroup.entrySet();
+						for (Entry<String, Map<CkanGroup, RolesCkanGroupOrOrg>> entry : set) {
+							Set<Entry<CkanGroup, RolesCkanGroupOrOrg>> subSet = entry.getValue().entrySet();
+							for (Entry<CkanGroup, RolesCkanGroupOrOrg> subEntry : subSet) {
+								toReturn.add(new OrganizationBean(subEntry.getKey().getTitle(), subEntry.getKey().getName(), false));
+							}
+						}
+						httpSession.setAttribute(keyPerScopeGroups, toReturn);
+					}else
+						logger.warn("The API_KEY for "+username+ " is null or empty in the catalogue: "+catalogue.getCatalogueUrl());
+				}catch(Exception e){
+					logger.error("Error on recovery the user groups for "+username, e);
 				}
-				httpSession.setAttribute(keyPerScopeGroups, toReturn);
 			}
 		}else{
 			logger.warn("Dev mode detected");
@@ -566,6 +666,13 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 	}
 
 
+	/**
+	 * Checks if is publisher user.
+	 *
+	 * @param isWorkspaceRequest the is workspace request
+	 * @return true, if is publisher user
+	 * @throws Exception the exception
+	 */
 	@Override
 	public boolean isPublisherUser(boolean isWorkspaceRequest) throws Exception{
 
@@ -605,7 +712,9 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 
 					// we build up also a list that keeps track of the scopes (orgs) in which the user has role ADMIN/EDITOR
 					List<OrganizationBean> orgsInWhichAtLeastEditorRole = new ArrayList<OrganizationBean>();
-					role = CatalogueRoleManager.getHighestRole(scopePerCurrentUrl, username, groupName, this, orgsInWhichAtLeastEditorRole);
+					String gatewayURL = GenericUtils.getGatewayClientHostname(getThreadLocalRequest());
+					logger.info("The Gateway URL is: "+gatewayURL);
+					role = CatalogueRoleManager.getHighestRole(scopePerCurrentUrl, username, groupName, this, orgsInWhichAtLeastEditorRole, gatewayURL);
 
 					// if he is an admin/editor preload:
 					// 1) organizations in which he can publish (the widget will find these info in session)
@@ -632,6 +741,13 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 		}
 	}
 
+	/**
+	 * Checks if is geo JSON valid.
+	 *
+	 * @param geoJson the geo json
+	 * @return true, if is geo JSON valid
+	 * @throws Exception the exception
+	 */
 	@Override
 	public boolean isGeoJSONValid(String geoJson) throws Exception {
 		try{
